package org.apidesign.apifest08.currency;

import java.io.Serializable;

/**
 * Value class, holding an exchange rate between two currencies.
 * Designed to be an immutable.
 * 
 * @author ked
 */
public final class ExchangeRateValue<AmountType, IdentifierType> implements Serializable {

    private final CurrencyValue<AmountType, IdentifierType> currencyA;
    private final CurrencyValue<AmountType, IdentifierType> currencyB;

    private ExchangeRateValue(
            CurrencyValue<AmountType, IdentifierType> currencyA,
            CurrencyValue<AmountType, IdentifierType> currencyB) {
        if (currencyA.getIdentifier() == null ||
            currencyB.getIdentifier() == null ||
            currencyA.getIdentifier().equals(currencyB)) {
                throw new IllegalArgumentException("Inappropriate exchange rates' identifiers!");
        }

        this.currencyA = currencyA;
        this.currencyB = currencyB;
    }

    public CurrencyValue<AmountType, IdentifierType> getCurrencyA() {
        return currencyA;
    }

    public CurrencyValue<AmountType, IdentifierType> getCurrencyB() {
        return currencyB;
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final ExchangeRateValue other = (ExchangeRateValue) obj;
        if (this.currencyA != other.currencyA && (this.currencyA == null || !this.currencyA.equals(other.currencyA))) {
            return false;
        }
        if (this.currencyB != other.currencyB && (this.currencyB == null || !this.currencyB.equals(other.currencyB))) {
            return false;
        }
        return true;
    }

    @Override
    public int hashCode() {
        int hash = 3;
        hash = 71 * hash + (this.currencyA != null ? this.currencyA.hashCode() : 0);
        hash = 71 * hash + (this.currencyB != null ? this.currencyB.hashCode() : 0);
        return hash;
    }

    /**
     * Creates new instance.
     * Generic types of the new instance are derived from types of the parameters.
     * 
     * @param <AmountType> type of the currency amount
     * @param <IdentifierType> type of the currency identifier
     * @param currencyA one currency of the exchange rate
     * @param currencyB another currency of the exchange rate
     * @return new instance
     */
    public static <AmountType, IdentifierType> ExchangeRateValue<AmountType, IdentifierType> getExchangeRate(
            CurrencyValue<AmountType, IdentifierType> currencyA,
            CurrencyValue<AmountType, IdentifierType> currencyB) {
        return new ExchangeRateValue<AmountType, IdentifierType>(currencyA, currencyB);
    }
}
