package org.apidesign.apifest08.test;

import java.util.Currency;

import junit.framework.TestCase;

import org.apidesign.apifest08.currency.Convertor;
import org.apidesign.apifest08.currency.exceptions.UnknownConvertorException;

/** There are many currencies around the world and many banks manipulate
 * with more than one or two at the same time. As banks are usually the
 * best paying clients, which is true even in case of your Convertor API,
 * it is reasonable to listen to their requests.
 * <p>
 * The quest for today is to enhance your existing convertor API to hold
 * information about many currencies and allow conversions between any of them.
 * Also, as conversion rates for diferent currencies usually arise from various
 * bank departments, there is another important need. There is a need to
 * compose two convertors into one by merging all the information about
 * currencies they know about.
 */
public class Task2Test extends TestCase {
    public Task2Test(String testName) {
        super(testName);
    }

    @Override
    protected void setUp() throws Exception {
    }

    @Override
    protected void tearDown() throws Exception {
    }

    // As in Task1Test, keep in mind, that there are three parts
    // of the whole system:
    // 1. there is someone who knows the current exchange rate
    // 2. there is someone who wants to do the conversion
    // 3. there is the API between 1. and 2. which allows them to communicate
    // 
    // Please backward compatibly enhance your existing API to support following
    // usecases:
    //
    
    /** Create convertor that understands two currencies, CZK and
     *  SKK. Make 100 SKK == 75 CZK. This is method for the group of users that
     *  knows the exchange rate, and needs to use the API to create objects
     *  with the exchange rate. Anyone shall be ready to call this method without
     *  any other method being called previously. The API itself shall know
     *  nothing about any rates, before this method is called.
     */
    public static Convertor createTripleConvertor() {
        // Rates: 1USD = 15CZK
        // Rates: 1USD = 20SKK
        // Rates: 75CZK = 100SKK
    	// set exchange rates
    	Convertor.setConvertorRates(Currency.getInstance("USD"), Currency.getInstance("CZK"), 15, 1);
    	Convertor.setConvertorRates(Currency.getInstance("USD"), Currency.getInstance("SKK"), 20, 1);
    	Convertor.setConvertorRates(Currency.getInstance("SKK"), Currency.getInstance("CZK"), 75, 100);
        
        // create new instance
        Convertor convertor = null;
        try {
          convertor = Convertor.getConvertorInstance(Currency.getInstance("USD"), Currency.getInstance("SKK"), Currency.getInstance("CZK"));
        } catch (UnknownConvertorException e) {
          e.printStackTrace();
        }
    	return convertor;
    }

    /** Define convertor that understands three currencies. Use it.
     */
    public void testConvertorForUSDandCZKandSKK() throws Exception {
        Convertor c = createTripleConvertor();

        // convert $5 to CZK using c:
        double result = c.convert(5d, Currency.getInstance("USD"), Currency.getInstance("CZK"));
        assertEquals("Result is not 75 CZK", 75.0, result);

        // convert $5 to SKK using c:
        result = c.convert(5d, Currency.getInstance("USD"), Currency.getInstance("SKK"));
        assertEquals("Result is not 100 SKK", 100.0, result);

        // convert 200SKK to CZK using c:
        result = c.convert(200d, Currency.getInstance("SKK"), Currency.getInstance("CZK"));
        assertEquals("Result is not 150 CZK", 150.0, result);

        // convert 200SKK to USK using c:
        result = c.convert(200d, Currency.getInstance("SKK"), Currency.getInstance("USD"));
        assertEquals("Result is not 10 USD", 10.0, result);
    }

    /** Merge all currency rates of convertor 1 with convertor 2.
     * Implement this using your API, preferably this method just delegates
     * into some API method which does the actual work, without requiring
     * API clients to code anything complex.
     */
    public static Convertor merge(Convertor one, Convertor two) {
        return one.merge(two);
    }

    /** Join the convertors from previous task, Task1Test and show that it
     * can be used to do reasonable conversions.
     */
    public void testConvertorComposition() throws Exception {
        Convertor c = merge(
            Task1Test.createCZKtoUSD(),
            Task1Test.createSKKtoCZK()
        );

        // convert $5 to CZK using c:
        double result = c.convert(5d, Currency.getInstance("USD"), Currency.getInstance("CZK"));
        assertEquals("Result is not 85 CZK", 85.0, result);

        // convert $8 to CZK using c:
        result = c.convert(8d, Currency.getInstance("USD"), Currency.getInstance("CZK"));
        assertEquals("Result is not 136 CZK", 136.0, result);

        // convert 1003CZK to USD using c:
        result = c.convert(1003d, Currency.getInstance("CZK"), Currency.getInstance("USD"));
        assertEquals("Result is not 59 USD", 59.0, result);

        // convert 16CZK using c:
        result = c.convert(16d, Currency.getInstance("CZK"), Currency.getInstance("SKK"));
        assertEquals("Result is not 20 SKK", 20.0, result);

        // convert 500SKK to CZK using c:
        result = c.convert(500d, Currency.getInstance("SKK"), Currency.getInstance("CZK"));
        assertEquals("Result is not 400 CZK", 400.0, result);

    }
}
