package org.apidesign.apifest08.currency;

import java.math.BigDecimal;
import java.util.Currency;
import java.util.Date;


/**
 * Create convertors using a flyweight to reduce the number of repetative creations of the same convertor.
 * 
 * @author D'Arcy Smith
 * @version 1.2
 */
public final class ConvertorFactory
{
    /** 
     * Prevent accidental construction.
     */
    private ConvertorFactory()
    {        
    }
    
    /**
     * Get the convertor for the specified currencies.  The currency name format
     * must be acceptable to java.util.Currency.getInstance(String)
     * 
     * @param a the currency to convert from.
     * @param aRate the exchange rate for a to b.
     * @param b the currency to convert to.
     * @param bRate the echante rate for b to a.
     * @return the convertor for the specified currencies.
     * @throws IllegalArgumentException if any of the arguments are null.
     */
    public static Convertor getConvertor(final String     a,
                                         final BigDecimal aRate,
                                         final String     b,
                                         final BigDecimal bRate)
    {
        final Currency  currencyA;
        final Currency  currencyB;
        final Convertor convertor;
        
        currencyA = Currency.getInstance(a);
        currencyB = Currency.getInstance(b);        
        convertor = getConvertor(currencyA, aRate, currencyB, bRate);
        
        return (convertor);
    }
    
    /**
     * Get the convertor for the specified currencies.
     * 
     * @param a the currency to convert from.
     * @param aRate the exchange rate for a to b.
     * @param b the currency to convert to.
     * @param bRate the echante rate for b to a.
     * @return the convertor for the specified currencies.
     * @throws IllegalArgumentException if either any of the arguments are null or if either rate <= 0.
     */
    public static Convertor getConvertor(final Currency   a,
                                         final BigDecimal aRate,
                                         final Currency   b,
                                         final BigDecimal bRate)
    {
        Convertor          convertor;
        final ExchangeRate rate;

        if(a == null)
        {
            throw new IllegalArgumentException("a cannot be null");
        }

        if(b == null)
        {
            throw new IllegalArgumentException("b cannot be null");
        }
        
        if(aRate == null)
        {
            throw new IllegalArgumentException("aRate cannot be null");
        }
        
        if(bRate == null)
        {
            throw new IllegalArgumentException("bRate cannot be null");
        }

        rate      = ExchangeRate.getExchangeRate(a, b, aRate, bRate);
        convertor = getConvertor(rate);
                
        return (convertor);
    }
    
    public static Convertor getConvertor(final ExchangeRate rate)
    {
        final ConvertorImpl convertor;

        if(rate == null)
        {
            throw new IllegalArgumentException("rate cannot be null");
        }
        
        convertor = new ConvertorImpl(rate);
                
        return (convertor);        
    }

    public static DatedConvertor getConvertor(final Date      from,
                                              final Date      till,
                                              final Convertor convertor)
    {
        final DateRange      range;
        final ExchangeRate   rate;
        final DatedConvertor datedConvertor;

        if(from == null)
        {
            throw new IllegalArgumentException("from cannot be null");
        }

        if(till == null)
        {
            throw new IllegalArgumentException("till cannot be null");
        }

        if(convertor == null)
        {
            throw new IllegalArgumentException("convertor cannot be null");
        }
        
        if(from.after(till))
        {
            throw new IllegalArgumentException(from + " cannot be after " + till);
        }
        
        if(convertor instanceof ExchangeRateConvertor)
        {
            rate = ((ExchangeRateConvertor)convertor).getExchangeRate();
        }
        else
        {
            throw new Error();
        }

        range          = new DateRange(from, till);
        datedConvertor = new DatedConvertorImpl(range, rate);
        
        return (datedConvertor);
    }

    /**
     * 
     * @param cs
     * @return
     */
    public static Convertor mergeConvertors(final Convertor ... cs)
    {
        Convertor convertor;
        int       dated;
        int       nonDated;

        dated    = 0;
        nonDated = 0;

        for(final Convertor c : cs)
        {
            if(c instanceof DatedConvertor)
            {
                dated++;
            }
            else
            {
                nonDated++;
            }
        }

        if(dated != 0 && nonDated != 0)
        {
            throw new IllegalArgumentException("cannot mix DatedConvertors and non-DatedConvertors");
        }

        if(dated != 0)
        {
            convertor = new DatedCompositeConvertorImpl(cs);
        }
        else
        {
            convertor = new CompositeConvertorImpl(cs);
        }

        return (convertor);
    }
}
